@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2022
+# © Copyright EnterpriseDB UK Limited 2018-2025
 #
 # This file is part of Barman.
 #
@@ -22,15 +22,15 @@ import os.path
 from contextlib import closing
 
 from barman.clients.cloud_cli import (
-    add_tag_argument,
-    create_argument_parser,
     CLIErrorExit,
     GeneralErrorExit,
     NetworkErrorExit,
     UrlArgumentType,
+    add_tag_argument,
+    create_argument_parser,
 )
-from barman.cloud import configure_logging
 from barman.clients.cloud_compression import compress
+from barman.cloud import configure_logging
 from barman.cloud_providers import get_cloud_interface
 from barman.exceptions import BarmanException
 from barman.utils import check_positive, check_size, force_str
@@ -132,8 +132,7 @@ def parse_arguments(args=None):
     compression.add_argument(
         "-z",
         "--gzip",
-        help="gzip-compress the WAL while uploading to the cloud "
-        "(should not be used with python < 3.2)",
+        help="gzip-compress the WAL while uploading to the cloud",
         action="store_const",
         const="gzip",
         dest="compression",
@@ -141,12 +140,18 @@ def parse_arguments(args=None):
     compression.add_argument(
         "-j",
         "--bzip2",
-        help="bzip2-compress the WAL while uploading to the cloud "
-        "(should not be used with python < 3.3)",
+        help="bzip2-compress the WAL while uploading to the cloud",
         action="store_const",
         const="bzip2",
         dest="compression",
     )
+    compression.add_argument(
+        "--xz",
+        help="xz-compress the WAL while uploading to the cloud",
+        action="store_const",
+        const="xz",
+        dest="compression",
+    )
     compression.add_argument(
         "--snappy",
         help="snappy-compress the WAL while uploading to the cloud "
@@ -155,6 +160,22 @@ def parse_arguments(args=None):
         const="snappy",
         dest="compression",
     )
+    compression.add_argument(
+        "--zstd",
+        help="zstd-compress the WAL while uploading to the cloud "
+        "(requires optional zstandard library)",
+        action="store_const",
+        const="zstd",
+        dest="compression",
+    )
+    compression.add_argument(
+        "--lz4",
+        help="lz4-compress the WAL while uploading to the cloud "
+        "(requires optional lz4 library)",
+        action="store_const",
+        const="lz4",
+        dest="compression",
+    )
     add_tag_argument(
         parser,
         name="tags",
@@ -165,6 +186,14 @@ def parse_arguments(args=None):
         name="history-tags",
         help="Tags to be added to archived history files in cloud storage",
     )
+    gcs_arguments = parser.add_argument_group(
+        "Extra options for google-cloud-storage cloud provider"
+    )
+    gcs_arguments.add_argument(
+        "--kms-key-name",
+        help="The name of the GCP KMS key which should be used for encrypting the "
+        "uploaded data in GCS.",
+    )
     s3_arguments.add_argument(
         "-e",
         "--encryption",
@@ -173,6 +202,12 @@ def parse_arguments(args=None):
         choices=["AES256", "aws:kms"],
         metavar="ENCRYPTION",
     )
+    s3_arguments.add_argument(
+        "--sse-kms-key-id",
+        help="The AWS KMS key ID that should be used for encrypting the uploaded data "
+        "in S3. Can be specified using the key ID on its own or using the full ARN for "
+        "the key. Only allowed if `-e/--encryption` is set to `aws:kms`.",
+    )
     azure_arguments.add_argument(
         "--encryption-scope",
         help="The name of an encryption scope defined in the Azure Blob Storage "
@@ -302,9 +337,21 @@ class CloudWalUploader(object):
             # add bz2 extension
             return "%s.bz2" % wal_name
 
+        elif self.compression == "xz":
+            # add xz extension
+            return "%s.xz" % wal_name
+
         elif self.compression == "snappy":
             # add snappy extension
             return "%s.snappy" % wal_name
+
+        elif self.compression == "zstd":
+            # add zst extension
+            return "%s.zst" % wal_name
+
+        elif self.compression == "lz4":
+            # add lz4 extension
+            return "%s.lz4" % wal_name
         else:
             raise ValueError("Unknown compression type: %s" % self.compression)
 
