@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2022
+# © Copyright EnterpriseDB UK Limited 2018-2025
 #
 # This file is part of Barman.
 #
@@ -22,17 +22,17 @@ import sys
 from contextlib import closing
 
 from barman.clients.cloud_cli import (
-    create_argument_parser,
     CLIErrorExit,
     GeneralErrorExit,
     NetworkErrorExit,
     OperationErrorExit,
+    create_argument_parser,
 )
-from barman.cloud import configure_logging, ALLOWED_COMPRESSIONS
+from barman.cloud import ALLOWED_COMPRESSIONS, configure_logging
 from barman.cloud_providers import get_cloud_interface
 from barman.exceptions import BarmanException
 from barman.utils import force_str
-from barman.xlog import hash_dir, is_any_xlog_file, is_backup_file
+from barman.xlog import hash_dir, is_any_xlog_file, is_backup_file, is_partial_file
 
 
 def main(args=None):
@@ -68,7 +68,7 @@ def main(args=None):
                 logging.error("Bucket %s does not exist", cloud_interface.bucket_name)
                 raise OperationErrorExit()
 
-            downloader.download_wal(config.wal_name, config.wal_dest)
+            downloader.download_wal(config.wal_name, config.wal_dest, config.no_partial)
 
     except Exception as exc:
         logging.error("Barman cloud WAL restore exception: %s", force_str(exc))
@@ -90,6 +90,12 @@ def parse_arguments(args=None):
         "Currently AWS S3, Azure Blob Storage and Google Cloud Storage are supported.",
     )
 
+    parser.add_argument(
+        "--no-partial",
+        help="Do not download partial WAL files",
+        action="store_true",
+        default=False,
+    )
     parser.add_argument(
         "wal_name",
         help="The value of the '%%f' keyword (according to 'restore_command').",
@@ -118,12 +124,13 @@ class CloudWalDownloader(object):
         self.cloud_interface = cloud_interface
         self.server_name = server_name
 
-    def download_wal(self, wal_name, wal_dest):
+    def download_wal(self, wal_name, wal_dest, no_partial):
         """
         Download a WAL file from cloud storage
 
         :param str wal_name: Name of the WAL file
         :param str wal_dest: Full path of the destination WAL file
+        :param bool no_partial: Do not download partial WAL files
         """
 
         # Correctly format the source path on s3
@@ -139,39 +146,44 @@ class CloudWalDownloader(object):
         remote_name = None
         # Automatically detect compression based on the file extension
         compression = None
-        for item in self.cloud_interface.list_bucket(source_dir):
+        for item in self.cloud_interface.list_bucket(wal_path):
             # perfect match (uncompressed file)
             if item == wal_path:
                 remote_name = item
+                continue
             # look for compressed files or .partial files
-            elif item.startswith(wal_path):
-                # Detect compression
-                basename = item
-                for e, c in ALLOWED_COMPRESSIONS.items():
-                    if item[-len(e) :] == e:
-                        # Strip extension
-                        basename = basename[: -len(e)]
-                        compression = c
-                        break
 
-                # Check basename is a known xlog file (.partial?)
-                if not is_any_xlog_file(basename):
-                    logging.warning("Unknown WAL file: %s", item)
-                    continue
-                # Exclude backup informative files (not needed in recovery)
-                elif is_backup_file(basename):
-                    logging.info("Skipping backup file: %s", item)
-                    continue
+            # Detect compression
+            basename = item
+            for e, c in ALLOWED_COMPRESSIONS.items():
+                if item[-len(e) :] == e:
+                    # Strip extension
+                    basename = basename[: -len(e)]
+                    compression = c
+                    break
 
-                # Found candidate
-                remote_name = item
-                logging.info(
-                    "Found WAL %s for server %s as %s",
-                    wal_name,
-                    self.server_name,
-                    remote_name,
-                )
-                break
+            # Check basename is a known xlog file (.partial?)
+            if not is_any_xlog_file(basename):
+                logging.warning("Unknown WAL file: %s", item)
+                continue
+            # Exclude backup informative files (not needed in recovery)
+            elif is_backup_file(basename):
+                logging.info("Skipping backup file: %s", item)
+                continue
+            # Exclude partial files if required
+            elif no_partial and is_partial_file(basename):
+                logging.info("Skipping partial file: %s", item)
+                continue
+
+            # Found candidate
+            remote_name = item
+            logging.info(
+                "Found WAL %s for server %s as %s",
+                wal_name,
+                self.server_name,
+                remote_name,
+            )
+            break
 
         if not remote_name:
             logging.info(
