@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2023
+# © Copyright EnterpriseDB UK Limited 2018-2025
 #
 # This file is part of Barman.
 #
@@ -72,12 +72,17 @@ def _get_azure_credential(credential_type):
         return None
 
     try:
-        from azure.identity import AzureCliCredential, ManagedIdentityCredential
+        from azure.identity import (
+            AzureCliCredential,
+            DefaultAzureCredential,
+            ManagedIdentityCredential,
+        )
     except ImportError:
         raise SystemExit("Missing required python module: azure-identity")
 
     supported_credentials = {
         "azure-cli": AzureCliCredential,
+        "default": DefaultAzureCredential,
         "managed-identity": ManagedIdentityCredential,
     }
     try:
@@ -197,7 +202,17 @@ def get_snapshot_interface(config):
     elif config.cloud_provider == "aws-s3":
         from barman.cloud_providers.aws_s3 import AwsCloudSnapshotInterface
 
-        return AwsCloudSnapshotInterface(config.aws_profile, config.aws_region)
+        args = [
+            config.aws_profile,
+            config.aws_region,
+            config.aws_await_snapshots_timeout,
+            config.aws_snapshot_lock_mode,
+            config.aws_snapshot_lock_duration,
+            config.aws_snapshot_lock_cool_off_period,
+            config.aws_snapshot_lock_expiration_date,
+            config.tags,
+        ]
+        return AwsCloudSnapshotInterface(*args)
     else:
         raise CloudProviderUnsupported(
             "No snapshot provider for cloud provider: %s" % config.cloud_provider
@@ -245,7 +260,13 @@ def get_snapshot_interface_from_server_config(server_config):
         from barman.cloud_providers.aws_s3 import AwsCloudSnapshotInterface
 
         return AwsCloudSnapshotInterface(
-            server_config.aws_profile, server_config.aws_region
+            server_config.aws_profile,
+            server_config.aws_region,
+            server_config.aws_await_snapshots_timeout,
+            server_config.aws_snapshot_lock_mode,
+            server_config.aws_snapshot_lock_duration,
+            server_config.aws_snapshot_lock_cool_off_period,
+            server_config.aws_snapshot_lock_expiration_date,
         )
     else:
         raise CloudProviderUnsupported(
@@ -315,9 +336,11 @@ def get_snapshot_interface_from_backup_info(backup_info, config=None):
         # config region takes precedence.
         region = None
         profile = None
-        if config is not None and hasattr(config, "aws_region"):
-            region = config.aws_region
-            profile = config.aws_profile
+        if config is not None:
+            if hasattr(config, "aws_region"):
+                region = config.aws_region
+            if hasattr(config, "aws_profile"):
+                profile = config.aws_profile
         if region is None:
             region = backup_info.snapshots_info.region
         return AwsCloudSnapshotInterface(profile, region)
@@ -343,15 +366,11 @@ def snapshots_info_from_dict(snapshots_info):
 
         return GcpSnapshotsInfo.from_dict(snapshots_info)
     elif "provider" in snapshots_info and snapshots_info["provider"] == "azure":
-        from barman.cloud_providers.azure_blob_storage import (
-            AzureSnapshotsInfo,
-        )
+        from barman.cloud_providers.azure_blob_storage import AzureSnapshotsInfo
 
         return AzureSnapshotsInfo.from_dict(snapshots_info)
     elif "provider" in snapshots_info and snapshots_info["provider"] == "aws":
-        from barman.cloud_providers.aws_s3 import (
-            AwsSnapshotsInfo,
-        )
+        from barman.cloud_providers.aws_s3 import AwsSnapshotsInfo
 
         return AwsSnapshotsInfo.from_dict(snapshots_info)
     else:
