@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -32,7 +32,6 @@ import signal
 import subprocess
 import sys
 import time
-
 from distutils.version import LooseVersion as Version
 
 import barman.utils
@@ -41,6 +40,22 @@ from barman.exceptions import CommandFailedException, CommandMaxRetryExceeded
 _logger = logging.getLogger(__name__)
 
 
+class Handler:
+    def __init__(self, logger, level, prefix=None):
+        self.class_logger = logger
+        self.level = level
+        self.prefix = prefix
+
+    def run(self, line):
+        if line:
+            if self.prefix:
+                self.class_logger.log(self.level, "%s%s", self.prefix, line)
+            else:
+                self.class_logger.log(self.level, "%s", line)
+
+    __call__ = run
+
+
 class StreamLineProcessor(object):
     """
     Class deputed to reading lines from a file object, using a buffered read.
@@ -165,7 +180,7 @@ class Command(object):
         the method arguments will take the precedence over
         the constructor arguments.
 
-        :param str cmd: The command to exexute
+        :param str cmd: The command to execute
         :param list[str]|None args: List of additional arguments to append
         :param dict[str.str]|None env_append: additional environment variables
         :param str path: PATH to be used while searching for `cmd`
@@ -219,7 +234,7 @@ class Command(object):
         if out_handler:
             self.out_handler = out_handler
         else:
-            self.out_handler = self.make_logging_handler(logging.INFO)
+            self.out_handler = self.make_logging_handler(logging.DEBUG)
         # If an error handler has been provided use it, otherwise log the
         # stderr as WARNING
         if err_handler:
@@ -340,11 +355,26 @@ class Command(object):
         """
         out = []
         err = []
+
+        def out_handler(line):
+            out.append(line)
+            if self.out_handler is not None:
+                self.out_handler(line)
+
+        def err_handler(line):
+            err.append(line)
+            if self.err_handler is not None:
+                self.err_handler(line)
+
         # If check is true, it must be handled here
         check = kwargs.pop("check", self.check)
         allowed_retval = kwargs.pop("allowed_retval", self.allowed_retval)
         self.execute(
-            out_handler=out.append, err_handler=err.append, check=False, *args, **kwargs
+            out_handler=out_handler,
+            err_handler=err_handler,
+            check=False,
+            *args,
+            **kwargs
         )
         self.out = "\n".join(out)
         self.err = "\n".join(err)
@@ -511,7 +541,7 @@ class Command(object):
         """
         Build a handler function that logs every line it receives.
 
-        The resulting function logs its input at the specified level
+        The resulting callable object logs its input at the specified level
         with an optional prefix.
 
         :param level: The log level to use
@@ -520,14 +550,7 @@ class Command(object):
         """
         class_logger = logging.getLogger(cls.__name__)
 
-        def handler(line):
-            if line:
-                if prefix:
-                    class_logger.log(level, "%s%s", prefix, line)
-                else:
-                    class_logger.log(level, "%s", line)
-
-        return handler
+        return Handler(class_logger, level, prefix)
 
     @staticmethod
     def make_output_handler(prefix=None):
@@ -679,7 +702,9 @@ class Rsync(Command):
         """
         if "stdin" in kwargs:
             raise TypeError("from_file_list() doesn't support 'stdin' keyword argument")
-        input_string = ("\n".join(filelist)).encode("UTF-8")
+        # The input string for the rsync --files-from argument must have a
+        # trailing newline for compatibility with certain versions of rsync.
+        input_string = ("\n".join(filelist) + "\n").encode("UTF-8")
         _logger.debug("from_file_list: %r", filelist)
         kwargs["stdin"] = input_string
         self.get_output("--files-from=-", src, dst, *args, **kwargs)
@@ -714,7 +739,7 @@ class PostgreSQLClient(Command):
     Sometimes the name of a command has been changed during the PostgreSQL
     evolution. I.e. that happened with pg_receivexlog, that has been renamed
     to pg_receivewal. In that case, we should try using pg_receivewal (the
-    newer auternative) and, if that command doesn't exist, we should try
+    newer alternative) and, if that command doesn't exist, we should try
     using `pg_receivexlog`.
 
     This is a list of command names to be used to find the installed command.
@@ -734,7 +759,10 @@ class PostgreSQLClient(Command):
         :param str path: additional path for executable retrieval
         """
         Command.__init__(self, command, path=path, **kwargs)
-
+        if not connection:
+            self.enable_signal_forwarding(signal.SIGINT)
+            self.enable_signal_forwarding(signal.SIGTERM)
+            return
         if version and version >= Version("9.3"):
             # If version of the client is >= 9.3 we use the connection
             # string because allows the user to use all the parameters
@@ -888,6 +916,8 @@ class PgBaseBackup(PostgreSQLClient):
         tbs_mapping=None,
         immediate=False,
         check=True,
+        compression=None,
+        parent_backup_manifest_path=None,
         args=None,
         **kwargs
     ):
@@ -905,6 +935,11 @@ class PgBaseBackup(PostgreSQLClient):
         :param bool immediate: fast checkpoint identifier for pg_basebackup
         :param bool check: check if the return value is in the list of
           allowed values of the Command obj
+        :param barman.compression.PgBaseBackupCompression compression:
+          the pg_basebackup compression options used for this backup
+        :param str parent_backup_manifest_path:
+          the path to a backup_manifest file from a previous backup which can
+          be used to perform an incremental backup
         :param List[str] args: additional arguments
         """
         PostgreSQLClient.__init__(
@@ -931,7 +966,7 @@ class PgBaseBackup(PostgreSQLClient):
 
         # The tablespace mapping option is repeated once for each tablespace
         if tbs_mapping:
-            for (tbs_source, tbs_destination) in tbs_mapping.items():
+            for tbs_source, tbs_destination in tbs_mapping.items():
                 self.args.append(
                     "--tablespace-mapping=%s=%s" % (tbs_source, tbs_destination)
                 )
@@ -940,14 +975,69 @@ class PgBaseBackup(PostgreSQLClient):
         if bwlimit is not None and bwlimit > 0:
             self.args.append("--max-rate=%s" % bwlimit)
 
+        # If it has a manifest file path it means it is an incremental backup
+        if parent_backup_manifest_path:
+            self.args.append("--incremental=%s" % parent_backup_manifest_path)
+
         # Immediate checkpoint
         if immediate:
             self.args.append("--checkpoint=fast")
 
+        # Append compression arguments, the exact format of which are determined
+        # in another function since they depend on the command version
+        self.args.extend(self._get_compression_args(version, compression))
+
         # Manage additional args
         if args:
             self.args += args
 
+    def _get_compression_args(self, version, compression):
+        """
+        Determine compression related arguments for pg_basebackup from the supplied
+        compression options in the format required by the pg_basebackup version.
+
+        :param Version version: The pg_basebackup version for which the arguments
+          should be formatted.
+        :param barman.compression.PgBaseBackupCompression compression:
+          the pg_basebackup compression options used for this backup
+        """
+        compression_args = []
+
+        if compression is not None:
+            if compression.config.format is not None:
+                compression_format = compression.config.format
+            else:
+                compression_format = "tar"
+            compression_args.append("--format=%s" % compression_format)
+            # For clients >= 15 we use the new --compress argument format
+            if version and version >= Version("15"):
+                compress_arg = "--compress="
+                detail = []
+                if compression.config.location is not None:
+                    compress_arg += "%s-" % compression.config.location
+                compress_arg += compression.config.type
+                if compression.config.level is not None:
+                    detail.append("level=%d" % compression.config.level)
+                if compression.config.workers is not None:
+                    detail.append("workers=%d" % compression.config.workers)
+                if detail:
+                    compress_arg += ":%s" % ",".join(detail)
+
+                compression_args.append(compress_arg)
+            # For clients < 15 we use the old style argument format
+            else:
+                if compression.config.type == "none":
+                    compression_args.append("--compress=0")
+                else:
+                    if compression.config.level is not None:
+                        compression_args.append(
+                            "--compress=%d" % compression.config.level
+                        )
+                    # --gzip must be positioned after --compress when compression level=0
+                    # so `base.tar.gz` can be created. Otherwise `.gz` won't be added.
+                    compression_args.append("--%s" % compression.config.type)
+        return compression_args
+
 
 class PgReceiveXlog(PostgreSQLClient):
     """
@@ -1014,6 +1104,109 @@ class PgReceiveXlog(PostgreSQLClient):
             self.args += args
 
 
+class PgVerifyBackup(PostgreSQLClient):
+    """
+    Wrapper class for the pg_verify system command
+    """
+
+    COMMAND_ALTERNATIVES = ["pg_verifybackup"]
+
+    def __init__(
+        self,
+        data_path,
+        command,
+        connection=None,
+        version=None,
+        app_name=None,
+        check=True,
+        args=None,
+        **kwargs
+    ):
+        """
+        Constructor
+        :param str data_path: backup data directory
+        :param str command: the command to use
+        :param PostgreSQL connection: an object representing
+          a database connection
+        :param Version version: the command version
+        :param str app_name: the application name to use for the connection
+        :param bool check: check if the return value is in the list of
+          allowed values of the Command obj
+        :param List[str] args: additional arguments
+        """
+        PostgreSQLClient.__init__(
+            self,
+            connection=connection,
+            command=command,
+            version=version,
+            app_name=app_name,
+            check=check,
+            **kwargs
+        )
+
+        self.args = ["-n", data_path]
+        if args:
+            self.args += args
+
+
+class PgCombineBackup(PostgreSQLClient):
+    """
+    Wrapper class for the ``pg_combinebackup`` system command
+    """
+
+    COMMAND_ALTERNATIVES = ["pg_combinebackup"]
+
+    def __init__(
+        self,
+        destination,
+        command,
+        tbs_mapping=None,
+        connection=None,
+        version=None,
+        app_name=None,
+        check=True,
+        args=None,
+        **kwargs
+    ):
+        """
+        Constructor
+
+        :param str destination: destination directory path
+        :param str command: the command to use
+        :param None|Dict[str, str] tbs_mapping: used for tablespace
+        :param PostgreSQL connection: an object representing
+          a database connection
+        :param Version version: the command version
+        :param str app_name: the application name to use for the connection
+        :param bool check: check if the return value is in the list of
+          allowed values of the :class:`Command` obj
+        :param None|List[str] args: additional arguments
+        """
+        PostgreSQLClient.__init__(
+            self,
+            connection=connection,
+            command=command,
+            version=version,
+            app_name=app_name,
+            check=check,
+            **kwargs
+        )
+
+        # Set the backup destination
+        self.args = ["--output=%s" % destination]
+
+        # The tablespace mapping option is repeated once for each tablespace
+        if tbs_mapping:
+            for tbs_source, tbs_destination in tbs_mapping.items():
+                self.args.append(
+                    "--tablespace-mapping=%s=%s" % (tbs_source, tbs_destination)
+                )
+
+        # Manage additional args
+        if args:
+            self.args += args
+
+
 class BarmanSubProcess(object):
     """
     Wrapper class for barman sub instances
@@ -1029,7 +1222,7 @@ class BarmanSubProcess(object):
     ):
         """
         Build a specific wrapper for all the barman sub-commands,
-        providing an unified interface.
+        providing a unified interface.
 
         :param str command: path to barman
         :param str subcommand: the barman sub-command
@@ -1111,6 +1304,8 @@ def shell_quote(arg):
     # a backslash, and then start another string using a quote character.
 
     assert arg is not None
+    if arg == "|":
+        return arg
     return "'%s'" % arg.replace("'", "'\\''")
 
 
