@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2023
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -506,7 +506,7 @@ class RsyncCopyController(object):
         """
         _logger.info(
             "Detected rsync version less than 3.1. "
-            "top using '--ignore-missing-args' argument."
+            "Stopping use of '--ignore-missing-args' argument."
         )
         self.rsync_has_ignore_missing_args = False
         self.rsync_cache.clear()
