@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2013-2023
+# © Copyright EnterpriseDB UK Limited 2013-2025
 #
 # This file is part of Barman.
 #
@@ -148,9 +148,12 @@ class UnixLocalCommand(object):
         if not self.exists(path):
             raise FsOperationFailed("Following path does not exist: %s" % path)
         args = ["-c", "%a", path]
-        if self.is_osx() or sys.platform.startswith("openbsd"):
+        if self.is_osx():
             print("is osx")
             args = ["-f", "%Lp", path]
+        if sys.platform.startswith("openbsd"):
+            print("is openbsd")
+            args = ["-f", "%Lp", path]
         cmd_ret = self.cmd("stat", args=args)
         if cmd_ret != 0:
             raise FsOperationFailed(
@@ -266,8 +269,12 @@ class UnixLocalCommand(object):
 
         self.cmd("uname", args=["-a"])
         result["kernel_ver"] = self.internal_cmd.out.rstrip()
-        self.cmd("python", args=["--version", "2>&1"])
-        result["python_ver"] = self.internal_cmd.out.rstrip()
+        result["python_ver"] = "Python %s.%s.%s" % (
+            sys.version_info.major,
+            sys.version_info.minor,
+            sys.version_info.micro,
+        )
+        result["python_executable"] = sys.executable
         self.cmd("rsync", args=["--version", "2>&1"])
         try:
             result["rsync_ver"] = self.internal_cmd.out.splitlines(True)[0].rstrip()
