@@ -1,4 +1,5 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# -*- coding: utf-8 -*-
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -24,33 +25,11 @@ import fcntl
 import os
 import re
 
-
-class LockFileException(Exception):
-    """
-    LockFile Exception base class
-    """
-    pass
-
-
-class LockFileBusy(LockFileException):
-    """
-    Raised when a lock file is not free
-    """
-    pass
-
-
-class LockFilePermissionDenied(LockFileException):
-    """
-    Raised when a lock file is not accessible
-    """
-    pass
-
-
-class LockFileParsingError(LockFileException):
-    """
-    Raised when the content of the lockfile is unexpected
-    """
-    pass
+from barman.exceptions import (
+    LockFileBusy,
+    LockFileParsingError,
+    LockFilePermissionDenied,
+)
 
 
 class LockFile(object):
@@ -77,7 +56,7 @@ class LockFile(object):
     """
 
     LOCK_PATTERN = None
-    """
+    r"""
     If defined in a subclass, it must be a compiled regular expression
     which matches the lock filename.
 
@@ -122,7 +101,7 @@ class LockFile(object):
         self.raise_if_fail = raise_if_fail
         self.wait = wait
 
-    def acquire(self, raise_if_fail=None, wait=None):
+    def acquire(self, raise_if_fail=None, wait=None, update_pid=True):
         """
         Creates and holds on to the lock file.
 
@@ -135,14 +114,16 @@ class LockFile(object):
 
         :param bool raise_if_fail: If True raise an exception on failure
         :param bool wait: If True issue a blocking request
+        :param bool update_pid: Whether to write our pid in the lockfile
         :returns bool: whether the lock has been acquired
         """
         if self.fd:
             return True
         fd = None
         # method arguments take precedence on class parameters
-        raise_if_fail = raise_if_fail \
-            if raise_if_fail is not None else self.raise_if_fail
+        raise_if_fail = (
+            raise_if_fail if raise_if_fail is not None else self.raise_if_fail
+        )
         wait = wait if wait is not None else self.wait
         try:
             # 384 is 0600 in octal, 'rw-------'
@@ -151,11 +132,12 @@ class LockFile(object):
             if not wait:
                 flags |= fcntl.LOCK_NB
             fcntl.flock(fd, flags)
-            # Once locked, replace the content of the file
-            os.lseek(fd, 0, os.SEEK_SET)
-            os.write(fd, ("%s\n" % os.getpid()).encode('ascii'))
-            # Truncate the file at the current position
-            os.ftruncate(fd, os.lseek(fd, 0, os.SEEK_CUR))
+            if update_pid:
+                # Once locked, replace the content of the file
+                os.lseek(fd, 0, os.SEEK_SET)
+                os.write(fd, ("%s\n" % os.getpid()).encode("ascii"))
+                # Truncate the file at the current position
+                os.ftruncate(fd, os.lseek(fd, 0, os.SEEK_CUR))
             self.fd = fd
             return True
         except (OSError, IOError) as e:
@@ -211,13 +193,13 @@ class LockFile(object):
         :raises LockFilePermissionDenied: when the lockfile is not accessible
         """
         try:
-            self.acquire(raise_if_fail=True, wait=False)
+            self.acquire(raise_if_fail=True, wait=False, update_pid=False)
         except LockFileBusy:
             try:
                 # Read the lock content and parse the PID
                 # NOTE: We cannot read it in the self.acquire method to avoid
                 # reading the previous locker PID
-                with open(self.filename, 'r') as file_object:
+                with open(self.filename, "r") as file_object:
                     return int(file_object.readline().strip())
             except ValueError as e:
                 # This should not happen
@@ -236,8 +218,8 @@ class GlobalCronLock(LockFile):
 
     def __init__(self, lock_directory):
         super(GlobalCronLock, self).__init__(
-            os.path.join(lock_directory, '.cron.lock'),
-            raise_if_fail=True)
+            os.path.join(lock_directory, ".cron.lock"), raise_if_fail=True
+        )
 
 
 class ServerBackupLock(LockFile):
@@ -250,8 +232,9 @@ class ServerBackupLock(LockFile):
 
     def __init__(self, lock_directory, server_name):
         super(ServerBackupLock, self).__init__(
-            os.path.join(lock_directory, '.%s-backup.lock' % server_name),
-            raise_if_fail=True)
+            os.path.join(lock_directory, ".%s-backup.lock" % server_name),
+            raise_if_fail=True,
+        )
 
 
 class ServerCronLock(LockFile):
@@ -264,8 +247,10 @@ class ServerCronLock(LockFile):
 
     def __init__(self, lock_directory, server_name):
         super(ServerCronLock, self).__init__(
-            os.path.join(lock_directory, '.%s-cron.lock' % server_name),
-            raise_if_fail=True, wait=False)
+            os.path.join(lock_directory, ".%s-cron.lock" % server_name),
+            raise_if_fail=True,
+            wait=False,
+        )
 
 
 class ServerXLOGDBLock(LockFile):
@@ -278,8 +263,10 @@ class ServerXLOGDBLock(LockFile):
 
     def __init__(self, lock_directory, server_name):
         super(ServerXLOGDBLock, self).__init__(
-            os.path.join(lock_directory, '.%s-xlogdb.lock' % server_name),
-            raise_if_fail=True, wait=True)
+            os.path.join(lock_directory, ".%s-xlogdb.lock" % server_name),
+            raise_if_fail=True,
+            wait=True,
+        )
 
 
 class ServerWalArchiveLock(LockFile):
@@ -292,8 +279,10 @@ class ServerWalArchiveLock(LockFile):
 
     def __init__(self, lock_directory, server_name):
         super(ServerWalArchiveLock, self).__init__(
-            os.path.join(lock_directory, '.%s-archive-wal.lock' % server_name),
-            raise_if_fail=True, wait=False)
+            os.path.join(lock_directory, ".%s-archive-wal.lock" % server_name),
+            raise_if_fail=True,
+            wait=False,
+        )
 
 
 class ServerWalReceiveLock(LockFile):
@@ -303,10 +292,84 @@ class ServerWalReceiveLock(LockFile):
     Creates a '.<SERVER>-receive-wal.lock' lock file under
     the given lock_directory for the named SERVER.
     """
+
     # TODO: Implement on the other LockFile subclasses
-    LOCK_PATTERN = re.compile(r'\.(?P<server_name>.+)-receive-wal\.lock')
+    LOCK_PATTERN = re.compile(r"\.(?P<server_name>.+)-receive-wal\.lock")
 
     def __init__(self, lock_directory, server_name):
         super(ServerWalReceiveLock, self).__init__(
-            os.path.join(lock_directory, '.%s-receive-wal.lock' % server_name),
-            raise_if_fail=True, wait=False)
+            os.path.join(lock_directory, ".%s-receive-wal.lock" % server_name),
+            raise_if_fail=True,
+            wait=False,
+        )
+
+
+class ServerBackupIdLock(LockFile):
+    """
+    This lock protects from changing a backup that is in use.
+
+    Creates a '.<SERVER>-<BACKUP_ID>.lock' lock file under the given
+    lock_directory for a BACKUP of a SERVER.
+    """
+
+    def __init__(self, lock_directory, server_name, backup_id):
+        super(ServerBackupIdLock, self).__init__(
+            os.path.join(lock_directory, ".%s-%s.lock" % (server_name, backup_id)),
+            raise_if_fail=True,
+            wait=False,
+        )
+
+
+class ServerBackupSyncLock(LockFile):
+    """
+    This lock protects from multiple executions of the sync command on the same
+    backup.
+
+    Creates a '.<SERVER>-<BACKUP>-sync-backup.lock' lock file under the given
+    lock_directory for a BACKUP of a SERVER.
+    """
+
+    def __init__(self, lock_directory, server_name, backup_id):
+        super(ServerBackupSyncLock, self).__init__(
+            os.path.join(
+                lock_directory, ".%s-%s-sync-backup.lock" % (server_name, backup_id)
+            ),
+            raise_if_fail=True,
+            wait=False,
+        )
+
+
+class ServerWalSyncLock(LockFile):
+    """
+    This lock protects from multiple executions of the sync-wal command
+
+    Creates a '.<SERVER>-sync-wal.lock' lock file under the given
+    lock_directory for the named SERVER.
+    """
+
+    def __init__(self, lock_directory, server_name):
+        super(ServerWalSyncLock, self).__init__(
+            os.path.join(lock_directory, ".%s-sync-wal.lock" % server_name),
+            raise_if_fail=True,
+            wait=True,
+        )
+
+
+class ConfigUpdateLock(LockFile):
+    """
+    This lock protects barman from multiple executions of config-update command
+
+    Creates a ``.config-update.lock`` lock file under the given ``lock_directory``.
+    """
+
+    def __init__(self, lock_directory):
+        """
+        Initialize a new :class:`ConfigUpdateLock` object.
+
+        :param lock_directory str: where to create the ``.config-update.lock`` file.
+        """
+        super(ConfigUpdateLock, self).__init__(
+            os.path.join(lock_directory, ".config-update.lock"),
+            raise_if_fail=True,
+            wait=False,
+        )
