@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2023
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -23,7 +23,6 @@ This module contains utility functions used in Barman.
 import datetime
 import decimal
 import errno
-from glob import glob
 import grp
 import hashlib
 import json
@@ -34,14 +33,15 @@ import pwd
 import re
 import signal
 import sys
-from argparse import ArgumentTypeError
 from abc import ABCMeta, abstractmethod
+from argparse import ArgumentTypeError
 from contextlib import contextmanager
+from distutils.version import Version
+from glob import glob
+
 from dateutil import tz
 
-from distutils.version import Version
-from barman import lockfile
-
+from barman import lockfile, xlog
 from barman.exceptions import TimeoutError
 
 _logger = logging.getLogger(__name__)
@@ -54,7 +54,16 @@ else:
     _text_type = unicode  # noqa
     _string_types = basestring  # noqa
 
-RESERVED_BACKUP_IDS = ("latest", "last", "oldest", "first", "last-failed")
+RESERVED_BACKUP_IDS = (
+    "latest",
+    "last",
+    "oldest",
+    "first",
+    "last-failed",
+    "latest-full",
+    "last-full",
+    "auto",
+)
 
 
 def drop_privileges(user):
@@ -570,25 +579,27 @@ def is_power_of_two(number):
     return number != 0 and (number & (number - 1)) == 0
 
 
-def file_md5(file_path, buffer_size=1024 * 16):
+def file_hash(file_path, buffer_size=1024 * 16, hash_algorithm="sha256"):
     """
-    Calculate the md5 checksum for the provided file path
+    Calculate the checksum for the provided file path with a specific
+    hashing algorithm
 
     :param str file_path: path of the file to read
     :param int buffer_size: read buffer size, default 16k
+    :param str hash_algorithm: sha256 | md5
     :return str: Hexadecimal md5 string
     """
-    md5 = hashlib.md5()
+    hash_func = hashlib.new(hash_algorithm)
     with open(file_path, "rb") as file_object:
         while 1:
             buf = file_object.read(buffer_size)
             if not buf:
                 break
-            md5.update(buf)
-    return md5.hexdigest()
+            hash_func.update(buf)
+    return hash_func.hexdigest()
 
 
-# Might be better to use stream instead of full file content. As done in file_md5.
+# Might be better to use stream instead of full file content. As done in file_hash.
 # Might create performance issue for large files.
 class ChecksumAlgorithm(with_metaclass(ABCMeta)):
     @abstractmethod
@@ -726,6 +737,89 @@ def check_positive(value):
     return int_value
 
 
+def check_aws_expiration_date_format(value):
+    """
+    Check user input for aws expiration date timestamp with a specific format.
+
+    :param value: str containing the value to check.
+    :raise ValueError: Fails with an invalid date.
+    """
+    fmt = "%Y-%m-%dT%H:%M:%S.%fZ"
+    try:
+        # Attempt to parse the input date string into a datetime object
+        return datetime.datetime.strptime(value, fmt)
+    except ValueError:
+        raise ArgumentTypeError(
+            "Invalid date: '%s'. Expected format is '%s'." % (value, fmt)
+        )
+
+
+def check_aws_snapshot_lock_duration_range(value):
+    """
+    Check for AWS Snapshot Lock duration range option
+
+    :param value: str containing the value to check
+    """
+    if value is None:
+        return None
+    try:
+        int_value = int(value)
+    except Exception:
+        raise ArgumentTypeError("'%s' is not a valid input" % value)
+
+    if not 1 <= int_value <= 36500:
+        raise ValueError(
+            "aws_snapshot_lock_duration must be between 1 and 36,500 days."
+        )
+
+    return int_value
+
+
+def check_aws_snapshot_lock_cool_off_period_range(value):
+    """
+    Check for AWS Snapshot Lock cool-off period range option
+
+    :param value: str containing the value to check
+    """
+    if value is None:
+        return None
+    try:
+        int_value = int(value)
+    except Exception:
+        raise ArgumentTypeError("'%s' is not a valid input" % value)
+
+    if not 1 <= int_value <= 72:
+        raise ValueError(
+            "aws_snapshot_lock_cool_off_period must be between 1 and 72 hours."
+        )
+
+    return int_value
+
+
+def check_aws_snapshot_lock_mode(value):
+    """
+    Replication slot names may only contain lower case letters, numbers,
+    and the underscore character. This function parse a replication slot name
+
+    :param str value: slot_name value
+    :return:
+    """
+
+    AWS_SNAPSHOT_LOCK_MODE = ["governance", "compliance"]
+    if value is None:
+        return None
+
+    value = value.lower()
+    if value not in AWS_SNAPSHOT_LOCK_MODE:
+        raise ValueError(
+            "Invalid AWS snapshot lock mode. "
+            "Please specify either 'governance' or 'compliance'. "
+            "Ensure that the mode you choose aligns with your snapshot locking "
+            "requirements."
+        )
+    return value
+
+
 def check_tli(value):
     """
     Check for a positive integer option, and also make "current" and "latest" acceptable values
@@ -864,6 +958,8 @@ def get_backup_id_using_shortcut(server, shortcut, BackupInfo):
         backup_id = server.get_first_backup_id()
     elif shortcut in ("last-failed"):
         backup_id = server.get_last_backup_id([BackupInfo.FAILED])
+    elif shortcut in ("latest-full", "last-full"):
+        backup_id = server.get_last_full_backup_id()
     elif is_backup_id(shortcut):
         backup_id = shortcut
 
@@ -957,3 +1053,135 @@ def edit_config(file, section, option, value, lines=None):
         lines.append("[" + section + "]\n")
         lines.append(option + " = " + value + "\n")
     return lines
+
+
+def get_backup_id_from_target_time(available_backups, target_time, target_tli=None):
+    """
+    Get backup ID from the catalog based on the recovery_target *target_time* and
+    *target_tli*.
+
+    :param list[BackupInfo] available_backups: List of BackupInfo objects.
+    :param str target_time: The target value with timestamp format
+        ``%Y-%m-%d %H:%M:%S`` with or without timezone.
+    :param int|None target_tli: Target timeline value, if a specific one is required.
+    :return str|None: ID of the backup.
+    :raises:
+        :exc:`ValueError`: If *target_time* is an invalid value.
+    """
+    from barman.infofile import load_datetime_tz
+
+    try:
+        parsed_target = load_datetime_tz(target_time)
+    except Exception as e:
+        raise ValueError(
+            "Unable to parse the target time parameter %r: %s" % (target_time, e)
+        )
+    for candidate_backup in sorted(
+        available_backups, key=lambda backup: backup.backup_id, reverse=True
+    ):
+        if candidate_backup.end_time <= parsed_target:
+            if target_tli is not None and candidate_backup.timeline != target_tli:
+                continue
+            return candidate_backup.backup_id
+    return None
+
+
+def get_backup_id_from_target_lsn(available_backups, target_lsn, target_tli=None):
+    """
+    Get backup ID from the catalog based on the recovery_target *target_lsn* and
+    *target_tli*.
+
+    :param list[BackupInfo] available_backups: List of BackupInfo objects.
+    :param str target_lsn: The target value with lsn format, e.g.,
+        ``3/64000000``.
+    :param int|None target_tli: Target timeline value, if a specific one is required.
+    :return str|None: ID of the backup.
+    """
+    parsed_target = xlog.parse_lsn(target_lsn)
+    for candidate_backup in sorted(
+        available_backups, key=lambda backup: backup.backup_id, reverse=True
+    ):
+        if xlog.parse_lsn(candidate_backup.end_xlog) <= parsed_target:
+            if target_tli is not None and candidate_backup.timeline != target_tli:
+                continue
+            return candidate_backup.backup_id
+    return None
+
+
+def get_last_backup_id(available_backups):
+    """
+    Get last backup ID from the catalog.
+
+    :param list[BackupInfo] available_backups: List of BackupInfo objects.
+    :return str|None: ID of the backup.
+    """
+    if len(available_backups) == 0:
+        return None
+
+    backups = sorted(available_backups, key=lambda backup: backup.backup_id)
+    return backups[-1].backup_id
+
+
+def get_backup_id_from_target_tli(available_backups, target_tli):
+    """
+    Get backup ID from the catalog based on the recovery_target *target_tli*.
+
+    :param list[BackupInfo] available_backups: Dict values of BackupInfo objects.
+    :param int target_tli: Target timeline value.
+    :return str|None: ID of the backup.
+    """
+    if len(available_backups) == 0:
+        return None
+
+    for candidate_backup in sorted(
+        available_backups, key=lambda backup: backup.backup_id, reverse=True
+    ):
+        if candidate_backup.timeline == target_tli:
+            return candidate_backup.backup_id
+    return None
+
+
+def parse_target_tli(obj, target_tli, backup_info=None):
+    """
+    Parse target timeline shorcut, ``latest`` and ``current``.
+
+    .. note::
+        This method is used in two other methods that are part of the recovery of a
+        a backup (:meth:`_set_pitr_targets` and :meth:`get_required_xlog_files`).
+        When called with a *backup_info*, it means that the recover operation uses a
+        specific backup for recovery and ``current`` is allowed in this case because
+        this backup is considered the ``current`` backup.
+        When called without a *backup_info*, it means that the recover operation is
+        going to fetch the best backup for recovery, so ``current`` is not allowed
+        because there is no ``current`` backup.
+
+        This method can also be used in the cloud script to retrieve the latest WAL
+        from the cloud catalog when shortcut is ``latest``.
+
+    :param BackupManager|CloudBackupCatalog obj: A BackupManager or a CloudBackupCatalog
+        object.
+    :param str|int target_tli: Target timeline value. Accepts both an integer
+        representing the timeline, or keywords accepted by Postgres, such as ``current``
+        and ``latest``.
+    :param None|BackupInfo backup_info: Backup info object.
+    :return int|None: ID of the timeline.
+    :raise ValueError: if *target_tli* is an invalid value.
+    """
+    parsed_target_tli = target_tli
+    if target_tli and type(target_tli) is str:
+        if target_tli == "current":
+            if backup_info is None:
+                raise ValueError(
+                    "'%s' is not a valid timeline keyword when recovering"
+                    " without a backup_id" % target_tli
+                )
+            else:
+                parsed_target_tli = backup_info.timeline
+        elif target_tli == "latest":
+            valid_timelines = obj.get_latest_archived_wals_info()
+            parsed_target_tli = int(max(valid_timelines.keys()), 16)
+        elif target_tli.isdigit():
+            parsed_target_tli = int(target_tli)
+        else:
+            raise ValueError("'%s' is not a valid timeline keyword" % target_tli)
+    return parsed_target_tli
