@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -17,16 +17,13 @@
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>
 
 import collections
-import datetime
-import errno
 import filecmp
 import logging
 import os
 import shutil
 from abc import ABCMeta, abstractmethod
-from glob import glob
-
 from distutils.version import LooseVersion as Version
+from glob import glob
 
 from barman import output, xlog
 from barman.command_wrappers import CommandFailedException, PgReceiveXlog
@@ -139,7 +136,6 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
         :param boolean verbose: Flag for verbose output
         """
         compressor = self.backup_manager.compression_manager.get_default_compressor()
-        stamp = datetime.datetime.utcnow().strftime("%Y%m%dT%H%M%SZ")
         processed = 0
         header = "Processing xlog segments from %s for %s" % (
             self.name,
@@ -216,19 +212,15 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                 os.unlink(wal_info.orig_filename)
                 continue
             except DuplicateWalFile:
+                self.server.move_wal_file_to_errors_directory(
+                    wal_info.orig_filename, wal_info.name, "duplicate"
+                )
                 output.info(
                     "\tError: %s is already present in server %s. "
                     "File moved to errors directory.",
                     wal_info.name,
                     self.config.name,
                 )
-                error_dst = os.path.join(
-                    self.config.errors_directory,
-                    "%s.%s.duplicate" % (wal_info.name, stamp),
-                )
-                # TODO: cover corner case of duplication (unlikely,
-                # but theoretically possible)
-                shutil.move(wal_info.orig_filename, error_dst)
                 continue
             except AbortedRetryHookScript as e:
                 _logger.warning(
@@ -275,14 +267,9 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                     self.name,
                     basename,
                 )
-                error_dst = os.path.join(
-                    self.config.errors_directory, "%s.%s.unknown" % (basename, stamp)
+                self.server.move_wal_file_to_errors_directory(
+                    error, basename, "unknown"
                 )
-                try:
-                    shutil.move(error, error_dst)
-                except IOError as e:
-                    if e.errno == errno.ENOENT:
-                        _logger.warning("%s not found" % error)
 
     def archive_wal(self, compressor, wal_info):
         """
@@ -478,7 +465,6 @@ class FileWalArchiver(WalArchiver):
     """
 
     def __init__(self, backup_manager):
-
         super(FileWalArchiver, self).__init__(backup_manager, "file archival")
 
     def fetch_remote_status(self):
